@@ -13,9 +13,16 @@
 from abc import ABC, abstractmethod
 from typing import Optional, Any, List, Dict
 
-from ethtx.models.decoded_model import DecodedCall, DecodedTransfer
-from ethtx.models.objects_model import Block, Transaction, Call, Event, TransactionMetadata, BlockMetadata
-from ethtx.providers.semantic_providers.semantics_repository import SemanticsRepository
+from ethtx.models.decoded_model import DecodedCall, DecodedTransfer, Proxy
+from ethtx.models.objects_model import (
+    Block,
+    Transaction,
+    Call,
+    Event,
+    TransactionMetadata,
+    BlockMetadata,
+)
+from ethtx.providers.semantic_providers import SemanticsRepository
 
 
 class ABIBasic:
@@ -42,11 +49,7 @@ class IABIDecoder(ABC, ABIBasic):
 
     @abstractmethod
     def decode_transaction(
-        self,
-        block: Block,
-        transaction: Transaction,
-        delegations: Dict[str, set],
-        token_proxies: Dict[str, dict],
+        self, block: Block, transaction: Transaction, proxies: Dict[str, Proxy]
     ):
         ...
 
@@ -56,8 +59,7 @@ class IABIDecoder(ABC, ABIBasic):
         call: Call,
         block: BlockMetadata,
         transaction: TransactionMetadata,
-        delegations: Dict[str, set],
-        token_proxies: Dict[str, dict]
+        proxies: Dict[str, Proxy],
     ) -> ABISubmoduleAbc.decode:
         ...
 
@@ -67,23 +69,18 @@ class IABIDecoder(ABC, ABIBasic):
         events: [Event],
         block: BlockMetadata,
         transaction: TransactionMetadata,
-        delegations: Dict[str, set],
-        token_proxies: Dict[str, dict],
+        proxies: Dict[str, Proxy],
     ) -> ABISubmoduleAbc.decode:
         ...
 
     @abstractmethod
     def decode_transfers(
-        self,
-        call: DecodedCall,
-        events: [Event],
-        token_proxies: Dict[str, dict]
+        self, call: DecodedCall, events: [Event], proxies: Dict[str, Proxy]
     ) -> ABISubmoduleAbc.decode:
         ...
 
     @abstractmethod
     def decode_balances(
-        self,
-        transfers: List[DecodedTransfer]
+        self, transfers: List[DecodedTransfer]
     ) -> ABISubmoduleAbc.decode:
         ...
